#!/usr/bin/perl -w

######################################################
#
# PROGRAM       : uhidledgui.pl
# USE           : NCurses interface to uhidled
# AUTHOR        : aleph
# DATE          : 23.03.09
#
# NOTES         : 
#
#####################################################


use strict;
use Getopt::Long;
use Curses::UI;


my ($PROG)        = ($0 =~ /\/*([^\/]+$)/);
my $debug;
my $pacdrive = 0;
my $controller = 1;

sub help {
        print "\n";
        print " $PROG: control uhidled\n";
        print "\n";
        print " $PROG\n";
        print "\n";
        print "   -d 		debug mode.\n";
        print "   -h 		help mode.\n";
        print "   -c [1-8]	controller\n";
        print "   -p 		pacdrive mode\n";
        
        print "\n";
        exit (1);
}

GetOptions( "help" 		=> \&help, "debug" => \$debug,
	"p|pacdrive"		=> \$pacdrive,
	"c|controller=i"	=> \$controller,
	
	) or help();

my $cui = Curses::UI->new(	-clear_on_exit => 1,
							-color_support => 1,
							-mouse_support => 0,
                            -debug => $debug, );
                            
my @menu = (
	{ -label => 'File', 
		-submenu => [
			{ -label => 'Exit      ^Q', -value => \&exit_dialog  }
			]
           },
        );                           
            
            my $menu = $cui->add(
                'menu','Menubar', 
                -menu => \@menu,
                -fg  => "red",
        );
        
my $win = $cui->add( 'win1', 'Window',
                             -border => 1,
                             -y    => 1,
                             -bfg => 'red',
                     );                           

$cui->set_binding(sub {$menu->focus()}, "\cX");
$cui->set_binding( \&exit_dialog , "\cQ");
    
my $label1 = $win->add(
	"label1", 'Label',
    -text     => "LED states, select to change",
    -bold     => 1,
    -x		   => 0,
    -y		   => 1,
    );
    
my $labelhex = $win->add(
	"labelhex", 'Label',
    -text     => "Current hex code:",
    -bold     => 0,
    -x		   => 0,
    -y		   => 6,
    );

my $labelhex2 = $win->add(
	"labelhex2", 'Label',
    -text     => "0x",
    -x		   => 18,
    -y		   => 6,
    );

my $texthex = $win->add(
	"texthex", 'TextEntry',
    -text     => "0000",
    -bold     => 1,
    -x		   => 20,
    -y		   => 6,
    -maxlength => 4, 
    -regexp 	=> '/^[0-9a-fA-F]*$/',
    -onChange	=> \&hex_changed,
    
    );

for(my $i=0;$i<16;$i++){    
    my $label = $win->add(
        "llabel$i", 'Label',
        -text     => sprintf("%2s",$i+1),
        -x		   => $i*4,
        -y		   => 3,
    );
}
      
my @checkboxs;

for(my $i=0;$i<16;$i++){    
    my $checkbox = $win->add(
        "mycheckbox$i", 'Checkbox',
        -label     => "",
        -checked   => 0,
        -x		   => $i*4,
        -y		   => 4,
        -onChange  => \&led_changed,
    );
	push @checkboxs, \$checkbox;
}

${$checkboxs[0]}->focus();

$cui->mainloop;

exit 0;

sub exit_dialog() {
	my $return = $cui->dialog(
		-message   => "Do you really want to quit?",
		-title     => "Are you sure???", 
		-buttons   => ['yes', 'no'],
	);

	exit(0) if $return;
}

sub led_changed() {
	# modify hex value
	my $val = 0;
	
	for(my $i=0;$i<16;$i++){
		${$checkboxs[$i]}->get() and $val += 1 << $i;
	}
	
	$texthex->text(sprintf('%04x',$val));
	
	#hex_changed(1);
}

sub hex_changed() {
	my ($widget,$skipcheck) = @_;
	
	my $val = $texthex->get();
	
	if(!$skipcheck){
		for(my $i=0;$i<16;$i++){
			if($val & (1 << $i)){
				${$checkboxs[$i]}->check();
			} else {
				${$checkboxs[$i]}->uncheck();
			}
		}
	}
	my $cmd = sprintf("uhidled -c %d %s -x %s", $controller, ($pacdrive ? "-p ":""), $val );
	system($cmd);
}

#----------------------------------------------------------------

